package icu.fur93.esp32_car

import android.bluetooth.BluetoothGatt
import android.bluetooth.BluetoothGattCharacteristic
import android.util.Log
import androidx.compose.runtime.State
import androidx.compose.runtime.mutableStateOf
import androidx.room.util.copy

class CarCommand {
    companion object {
        // 数据包头尾定义
        const val PACKET_T_HEAD = 0x00u
        const val PACKET_T_TAIL = 0xFFu
        const val PACKET_R_HEAD = 0x01u
        const val PACKET_R_TAIL = 0xFEu
        const val PACKET_MAX_LENGTH = 32u // 数据包最大长度

        // 指令定义
        const val CMD_GET_BT_STATUS = 0x10u
        const val CMD_GET_SPIFFS_STATUS = 0x11u
        const val CMD_GET_DISTANCE = 0x12u
        const val CMD_MOTOR_MOVE_CONTROL = 0x20u
        const val CMD_MOTOR_STEER_CONTROL = 0x21u
        const val CMD_MOTOR_SINGLE_CONTROL = 0x22u
        const val CMD_MOTOR_ROTATE_CONTROL = 0x23u
        const val CMD_MOTOR_XYR_CONTROL = 0x24u
        const val CMD_DEMO_PID = 0xf0u
        const val CMD_DEMO_PATH = 0xf1u

        const val CMD_STATUS_MOTOR = 0xE0u
    }
}

data class MotorState(
    var pwm: UInt = 0u,
    var in1: UInt = 0u,
    var in2: UInt = 0u
)

data class CarState(
    var controlState: CarControlState = CarControlState(),
    var motorAState: MotorState = MotorState(),
    var motorBState: MotorState = MotorState(),
    var motorCState: MotorState = MotorState(),
    var motorDState: MotorState = MotorState()
)

enum class LogDirection {
    SEND,    // 发送
    RECEIVE  // 接收
}


// 添加日志数据类
data class LogEntry(
    val timestamp: Long = System.currentTimeMillis(),
    val direction: LogDirection,  // "发送" 或 "接收"
    val data: String
)

class CarController(private val onStateChange: () -> Unit) {
    // 使用 MutableState 来存储小车状态
    private val _carState = mutableStateOf(CarState())
    val carState: State<CarState> = _carState
    
    var bluetoothGatt: BluetoothGatt? = null
    var rxCharacteristic: BluetoothGattCharacteristic? = null

    // 添加日志列表状态
    private val _logs = mutableStateOf<List<LogEntry>>(emptyList())
    val logs: State<List<LogEntry>> = _logs
    
    // 更新电机状态的方法
    private fun updateMotorState(
        motorA: MotorState? = null,
        motorB: MotorState? = null,
        motorC: MotorState? = null,
        motorD: MotorState? = null
    ) {
        _carState.value.motorAState = motorA ?: _carState.value.motorAState
        _carState.value.motorBState = motorB ?: _carState.value.motorBState
        _carState.value.motorCState = motorC ?: _carState.value.motorCState
        _carState.value.motorDState = motorD ?: _carState.value.motorDState
    }

    private fun sendCommand(command: ByteArray) {
        rxCharacteristic?.let { characteristic ->
            characteristic.value = command
            bluetoothGatt?.writeCharacteristic(characteristic)
            
            // 添加发送日志
            _logs.value = _logs.value + LogEntry(
                direction = LogDirection.SEND,
                data = command.joinToString(" ") { "0x%02X".format(it) }
            )
        }
    }

    fun moveForward(speed: Int = 255) {
        sendCommand(byteArrayOf(CarCommand.PACKET_T_HEAD.toByte(), 0x06, CarCommand.CMD_MOTOR_MOVE_CONTROL.toByte(), 0x01, speed.toByte(), CarCommand.PACKET_T_TAIL.toByte()))
    }

    fun moveBackward(speed: Int = 255) {
        sendCommand(byteArrayOf(CarCommand.PACKET_T_HEAD.toByte(), 0x06, CarCommand.CMD_MOTOR_MOVE_CONTROL.toByte(), 0x02, speed.toByte(), CarCommand.PACKET_T_TAIL.toByte()))
    }

    fun turnLeft(speed: Int = 255) {
        sendCommand(byteArrayOf(CarCommand.PACKET_T_HEAD.toByte(), 0x06, CarCommand.CMD_MOTOR_STEER_CONTROL.toByte(), 0x00, speed.toByte(), CarCommand.PACKET_T_TAIL.toByte()))
    }

    fun turnRight(speed: Int = 255) {
        sendCommand(byteArrayOf(CarCommand.PACKET_T_HEAD.toByte(), 0x06, CarCommand.CMD_MOTOR_STEER_CONTROL.toByte(), 0x01, speed.toByte(), CarCommand.PACKET_T_TAIL.toByte()))
    }

    fun stop() {
        sendCommand(byteArrayOf(CarCommand.PACKET_T_HEAD.toByte(), 0x06, CarCommand.CMD_MOTOR_MOVE_CONTROL.toByte(), 0x00, 0x00, CarCommand.PACKET_T_TAIL.toByte()))
    }

    fun sendXYR(x: Int, y: Int, r: Int) {
        sendCommand(byteArrayOf(CarCommand.PACKET_T_HEAD.toByte(), 0x07, CarCommand.CMD_MOTOR_XYR_CONTROL.toByte(), x.toByte(), y.toByte(), r.toByte(), CarCommand.PACKET_T_TAIL.toByte()))
    }

    fun onReceivePacket(packet: ByteArray) {
        // 添加接收日志
        _logs.value = _logs.value + LogEntry(
            direction = LogDirection.RECEIVE,
            data = packet.joinToString(" ") { "0x%02X".format(it) }
        )

        // 判断数据包格式时使用 toUByte()
        if (packet[0].toUByte() == CarCommand.PACKET_R_HEAD.toUByte() && 
            packet.size == packet[1].toUByte().toInt() && 
            packet[packet[1].toUByte().toInt() - 1].toUByte() == CarCommand.PACKET_R_TAIL.toUByte()) {
            val command = packet[2].toUByte()
            val data = packet.sliceArray(3 until packet[1].toUByte().toInt() - 1)

            Log.d("CarController", "command: 0x%02X".format(command.toInt() and 0xFF))

            // 解析数据包
            when (command.toUInt()) {
                // `01 0C E0 电机A_IN_1_2 电机A_PWM 电机B_IN_1_2 电机B_PWM 电机C_IN_1_2 电机C_PWM 电机D_IN_1_2 电机D_PWM FE`
                CarCommand.CMD_STATUS_MOTOR -> {

                    Log.d("CarController", "status motor data: ${data.joinToString(" ") { "0x%02X".format(it) }}")

                    // 创建新的状态对象
                    _carState.value = _carState.value.copy(
                        motorAState = MotorState(
                            pwm = data[1].toUByte().toUInt(),
                            in1 = data[0].toUByte().toUInt() and 1u,
                            in2 = (data[0].toUByte().toUInt() shr 1) and 1u
                        ),
                        motorBState = MotorState(
                            pwm = data[3].toUByte().toUInt(),
                            in1 = data[2].toUByte().toUInt() and 1u,
                            in2 = (data[2].toUByte().toUInt() shr 1) and 1u
                        ),
                        motorCState = MotorState(
                            pwm = data[5].toUByte().toUInt(),
                            in1 = data[4].toUByte().toUInt() and 1u,
                            in2 = (data[4].toUByte().toUInt() shr 1) and 1u
                        ),
                        motorDState = MotorState(
                            pwm = data[7].toUByte().toUInt(),
                            in1 = data[6].toUByte().toUInt() and 1u,
                            in2 = (data[6].toUByte().toUInt() shr 1) and 1u
                        )
                    )
                }
            }
        }
    }

    // 清除日志
    fun clearLogs() {
        _logs.value = emptyList()
    }
}

class CarControlState {
    public var speed: UInt = 0u
    public var direction: UInt = 0u
}
