package icu.fur93.esp32_car

import android.Manifest
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothGatt
import android.bluetooth.BluetoothGattCallback
import android.bluetooth.BluetoothGattCharacteristic
import android.bluetooth.BluetoothGattDescriptor
import android.bluetooth.le.BluetoothLeScanner
import android.bluetooth.le.ScanCallback
import android.bluetooth.le.ScanFilter
import android.bluetooth.le.ScanResult
import android.bluetooth.le.ScanSettings
import android.content.pm.ActivityInfo
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.os.ParcelUuid
import android.util.Log
import android.view.WindowManager
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.graphicsLayer
import androidx.compose.ui.unit.dp
import androidx.core.app.ActivityCompat
import androidx.core.view.WindowCompat
import androidx.core.view.WindowInsetsCompat
import androidx.core.view.WindowInsetsControllerCompat
import icu.fur93.esp32_car.ui.component.Joystick
import icu.fur93.esp32_car.ui.component.JoystickState
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import java.util.UUID
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale

class MainActivity : ComponentActivity() {
    private val serviceUUID = "6E400001-B5A3-F393-E0A9-E50E24DCCA9E"
    private val rxCharUUID = "6E400002-B5A3-F393-E0A9-E50E24DCCA9E"
    private val txCharUUID = "6E400003-B5A3-F393-E0A9-E50E24DCCA9E"

    private var bleScanner: BluetoothLeScanner? = null

    // 将deviceList移动到Activity作用域内并使用MutableList
    private val deviceList = mutableListOf<ScanResult>()

    // 添加可观察的连接状态
    private val _isConnected = mutableStateOf(false)
    val isConnected: State<Boolean> = _isConnected

    private val scanCallback = object : ScanCallback() {
        override fun onScanResult(callbackType: Int, result: ScanResult) {
            super.onScanResult(callbackType, result)
            Log.d("MainActivity", "onScanResult: ${result.device.name} - ${result.device.address}")
            if (!deviceList.any { it.device.address == result.device.address }) {
                deviceList.add(result)
            }
        }

        override fun onScanFailed(errorCode: Int) {
            super.onScanFailed(errorCode)
            Log.e("MainActivity", "扫描失败，错误码: $errorCode")
        }
    }

    private val carController by lazy {
        CarController { }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        // 设置全屏显示
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
            window.attributes.layoutInDisplayCutoutMode =
                WindowManager.LayoutParams.LAYOUT_IN_DISPLAY_CUTOUT_MODE_SHORT_EDGES
        }

        // 隐藏状态栏和导航栏
        WindowInsetsControllerCompat(window, window.decorView).let { controller ->
            controller.hide(WindowInsetsCompat.Type.systemBars())
            controller.systemBarsBehavior =
                WindowInsetsControllerCompat.BEHAVIOR_SHOW_TRANSIENT_BARS_BY_SWIPE
        }

        // 设置横屏
        requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE

        // 检查并请求所有必要的权限
        val requiredPermissions = arrayOf(
            Manifest.permission.BLUETOOTH_SCAN,
            Manifest.permission.BLUETOOTH_CONNECT,
            Manifest.permission.ACCESS_FINE_LOCATION,
            Manifest.permission.ACCESS_COARSE_LOCATION
        )

        val missingPermissions = requiredPermissions.filter {
            ActivityCompat.checkSelfPermission(this, it) != PackageManager.PERMISSION_GRANTED
        }

        if (missingPermissions.isNotEmpty()) {
            ActivityCompat.requestPermissions(
                this,
                missingPermissions.toTypedArray(),
                BLUETOOTH_PERMISSION_REQUEST_CODE
            )
        } else {
            // 如果已经有所有权限，直接开始扫描
            val bluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
            bleScanner = bluetoothAdapter?.bluetoothLeScanner
            startBleScan()
        }

        setContent {
            GamepadScreen(
                deviceList = deviceList,
                isConnected = isConnected.value,
                onStartScan = ::startBleScan,
                onConnect = ::connectToDevice,
                carController = carController
            )
        }
    }

    override fun onWindowFocusChanged(hasFocus: Boolean) {
        super.onWindowFocusChanged(hasFocus)
        if (hasFocus) {
            // 重新应用全屏设置
            WindowInsetsControllerCompat(window, window.decorView).let { controller ->
                controller.hide(WindowInsetsCompat.Type.systemBars())
                controller.systemBarsBehavior =
                    WindowInsetsControllerCompat.BEHAVIOR_SHOW_TRANSIENT_BARS_BY_SWIPE
            }
        }
    }

    private fun connectToDevice(result: ScanResult) {
        if (ActivityCompat.checkSelfPermission(
                this,
                Manifest.permission.BLUETOOTH_CONNECT
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(
                this,
                arrayOf(Manifest.permission.BLUETOOTH_CONNECT),
                BLUETOOTH_PERMISSION_REQUEST_CODE
            )
            return
        }

        result.device.connectGatt(this, false, object : BluetoothGattCallback() {
            override fun onConnectionStateChange(gatt: BluetoothGatt?, status: Int, newState: Int) {
                super.onConnectionStateChange(gatt, status, newState)
                when (newState) {
                    BluetoothGatt.STATE_CONNECTED -> {
                        carController.bluetoothGatt = gatt
                        _isConnected.value = true
                        gatt?.discoverServices()
                    }

                    BluetoothGatt.STATE_DISCONNECTED -> {
                        _isConnected.value = false
                        carController.bluetoothGatt = null
                    }
                }
            }

            override fun onServicesDiscovered(gatt: BluetoothGatt?, status: Int) {
                super.onServicesDiscovered(gatt, status)
                if (status == BluetoothGatt.GATT_SUCCESS) {
                    gatt?.getService(UUID.fromString(serviceUUID))?.let { service ->
                        carController.bluetoothGatt = gatt
                        carController.rxCharacteristic =
                            service.getCharacteristic(UUID.fromString(rxCharUUID))

                        // 注册通知监听器
                        val txCharacteristic =
                            service.getCharacteristic(UUID.fromString(txCharUUID))
                        gatt.setCharacteristicNotification(txCharacteristic, true)
                        txCharacteristic.descriptors.forEach { descriptor ->
                            descriptor.value = BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE
                            gatt.writeDescriptor(descriptor)
                        }
                    }
                }
            }

            override fun onCharacteristicChanged(
                gatt: BluetoothGatt,
                characteristic: BluetoothGattCharacteristic,
                value: ByteArray
            ) {
                super.onCharacteristicChanged(gatt, characteristic, value)
                if (characteristic.uuid == UUID.fromString(txCharUUID)) {
                    carController.onReceivePacket(value)
                }
            }
        })
    }

    private fun startBleScan() {
        if (ActivityCompat.checkSelfPermission(
                this,
                Manifest.permission.BLUETOOTH_SCAN
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(
                this,
                arrayOf(
                    Manifest.permission.BLUETOOTH_SCAN,
                    Manifest.permission.ACCESS_FINE_LOCATION
                ),
                BLUETOOTH_PERMISSION_REQUEST_CODE
            )
            return
        }

        // 先停止当前扫描
        try {
            bleScanner?.stopScan(scanCallback)
        } catch (e: Exception) {
            Log.e("MainActivity", "停止扫描失败: ${e.message}")
        }

        // 清空之前的扫描结果
        deviceList.clear()

        Log.d("MainActivity", "开始扫描...")

        // 添加UUID过滤器
        val scanFilter = ScanFilter.Builder()
            .setServiceUuid(ParcelUuid.fromString(serviceUUID))
            .build()

        val scanSettings = ScanSettings.Builder()
            .setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY)
            .build()

        try {
//            bleScanner?.startScan(listOf(scanFilter), scanSettings, scanCallback)
            bleScanner?.startScan(scanCallback)
        } catch (e: Exception) {
            Log.e("MainActivity", "开始扫描失败: ${e.message}")
        }

        // 10秒后停止扫描
        Handler(Looper.getMainLooper()).postDelayed({
            stopBleScan()
            Log.d("MainActivity", "停止扫描，发现 ${deviceList.size} 个设备")
        }, 10000)
    }

    private fun stopBleScan() {
        if (ActivityCompat.checkSelfPermission(
                this,
                Manifest.permission.BLUETOOTH_SCAN
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            return
        }
        try {
            bleScanner?.stopScan(scanCallback)
        } catch (e: Exception) {
            Log.e("MainActivity", "停止扫描失败: ${e.message}")
        }
    }

    // 添加权限请求结果处理
    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == BLUETOOTH_PERMISSION_REQUEST_CODE) {
            if (grantResults.all { it == PackageManager.PERMISSION_GRANTED }) {
                Log.d("MainActivity", "所有权限已获得")
                // 权限获得后重新初始化蓝牙
                val bluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
                bleScanner = bluetoothAdapter?.bluetoothLeScanner
            } else {
                Log.e("MainActivity", "部分权限未获得")
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        // 确保在Activity销毁时停止扫描
        stopBleScan()
    }

    companion object {
        private const val BLUETOOTH_PERMISSION_REQUEST_CODE = 1
    }
}

@Composable
fun ControlJoystick(
    modifier: Modifier,
    carController: CarController,
    joystickState: JoystickState,
    onJoystickStateChange: (JoystickState) -> Unit
) {
    // 记住上一次发送的状态和时间
    val lastSentState = remember { mutableStateOf(JoystickState()) }
    val lastSentTime = remember { mutableStateOf(0L) }
    
    // 用于停止命令重发的计数器
    val stopCommandCount = remember { mutableStateOf(0) }
    val coroutineScope = rememberCoroutineScope()

    Joystick(
        modifier = modifier,
        size = 350f
    ) { state ->
        // 更新状态
        onJoystickStateChange(state)
        
        val currentTime = System.currentTimeMillis()
        
        // 检查是否是停止状态（摇杆回正）
        val isStopCommand = state.x == 0f && state.y == 0f
        
        // 如果是停止命令，启动重发机制
        // 否则检查时间间隔和状态变化
        if (isStopCommand) {
            if (stopCommandCount.value == 0) {
                // 第一次发送停止命令
                carController.stop()
                stopCommandCount.value = 1
                
                // 启动延时重发
                coroutineScope.launch {
                    delay(50) // 延时50ms
                    carController.stop() // 再次发送停止命令
                    stopCommandCount.value = 2
                    
                    delay(50) // 再次延时50ms
                    carController.stop() // 第三次发送停止命令
                    stopCommandCount.value = 0 // 重置计数器
                }
            }
        } else if ((state.x != lastSentState.value.x || state.y != lastSentState.value.y) &&
                  currentTime - lastSentTime.value >= 50) {
            // 非停止命令的正常处理
            stopCommandCount.value = 0 // 重置停止命令计数器
            val x = (state.x * 100).toInt()
            val y = (state.y * 100).toInt()
            carController.sendXYR(x, y, 0)
            
            // 更新上次发送的状态和时间
            lastSentState.value = state
            lastSentTime.value = currentTime
        }
    }
}

@Composable
fun GamepadScreen(
    deviceList: List<ScanResult>,
    isConnected: Boolean,
    onStartScan: () -> Unit,
    onConnect: (ScanResult) -> Unit,
    carController: CarController
) {
    var showDeviceList by remember { mutableStateOf(false) }
    var sliderValue by remember { mutableStateOf(0f) }
    var joystickState by remember { mutableStateOf(JoystickState()) }

    Box(modifier = Modifier.fillMaxSize()) {
        Column(modifier = Modifier.fillMaxWidth()) {
            // 电机状态显示区域
            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(8.dp),
                horizontalArrangement = Arrangement.SpaceEvenly
            ) {
                // 摇杆状态显示
                Column {
                    Text("摇杆")
                    Text("X: %.2f".format(joystickState.x))
                    Text("Y: %.2f".format(joystickState.y))
                }

                // 电机A状态
                Column {
                    Text("电机A")
                    Text("PWM: ${carController.carState.value.motorAState.pwm}")
                    Text("IN1: ${carController.carState.value.motorAState.in1}")
                    Text("IN2: ${carController.carState.value.motorAState.in2}")
                }

                // 电机B状态
                Column {
                    Text("电机B")
                    Text("PWM: ${carController.carState.value.motorBState.pwm}")
                    Text("IN1: ${carController.carState.value.motorBState.in1}")
                    Text("IN2: ${carController.carState.value.motorBState.in2}")
                }

                // 电机C状态
                Column {
                    Text("电机C")
                    Text("PWM: ${carController.carState.value.motorCState.pwm}")
                    Text("IN1: ${carController.carState.value.motorCState.in1}")
                    Text("IN2: ${carController.carState.value.motorCState.in2}")
                }

                // 电机D状态
                Column {
                    Text("电机D")
                    Text("PWM: ${carController.carState.value.motorDState.pwm}")
                    Text("IN1: ${carController.carState.value.motorDState.in1}")
                    Text("IN2: ${carController.carState.value.motorDState.in2}")
                }

                // 设备选择按钮
                Button(
                    onClick = { showDeviceList = true },
                ) {
                    Text(if (isConnected) "已连接" else "选择设备")
                }
            }

            // 方向控制布局
            Row(
                modifier = Modifier.fillMaxSize(),
                horizontalArrangement = Arrangement.SpaceEvenly,
                verticalAlignment = Alignment.CenterVertically
            ) {
                // 左侧控制区
                Column(
                    horizontalAlignment = Alignment.CenterHorizontally,
                    verticalArrangement = Arrangement.Center,
                    modifier = Modifier
                        .weight(1f)
                        .fillMaxHeight()
                ) {
                    Row(
                        horizontalArrangement = Arrangement.Center,
                        verticalAlignment = Alignment.CenterVertically,
                        modifier = Modifier
                            .fillMaxWidth()
                            .padding(vertical = 16.dp)
                    ) {
                        ControlJoystick(
                            modifier = Modifier.padding(16.dp),
                            carController = carController,
                            joystickState = joystickState,
                            onJoystickStateChange = { newState ->
                                joystickState = newState
                            }
                        )
                    }
                }

                // 中间 Log 区
                // 用于显示发出的蓝牙指令
                LazyColumn(
                    modifier = Modifier
                        .fillMaxSize()
                        .padding(4.dp)
                        .weight(1f),
                    reverseLayout = true // 反转布局顺序
                ) {
                    items(carController.logs.value.filter { it.direction == LogDirection.SEND }.reversed()) { log ->
                        Row(
                            modifier = Modifier
                                .fillMaxWidth()
                                .padding(vertical = 2.dp),
                            horizontalArrangement = Arrangement.SpaceBetween
                        ) {
                            Text(
                                text = "[${SimpleDateFormat("HH:mm:ss.SSS", Locale.getDefault())
                                    .format(Date(log.timestamp))}]",
                                style = MaterialTheme.typography.bodySmall
                            )
                            Text(
                                text = log.data,
                                style = MaterialTheme.typography.bodySmall,
                                modifier = Modifier.weight(1f)
                            )
                        }
                    }
                }

                // 右侧控制区
                Column(
                    horizontalAlignment = Alignment.CenterHorizontally,
                    verticalArrangement = Arrangement.Center,
                    modifier = Modifier.weight(1f)
                ) {
                    // 垂直滑杆
                    Slider(
                        value = sliderValue,
                        onValueChange = { newValue ->
                            // 在0附近添加吸附效果
                            val snapValue = when {
                                newValue > -0.1f && newValue < 0.1f -> 0f
                                else -> newValue
                            }
                            sliderValue = snapValue
                            val speed = (snapValue * 255).toInt()
                            if (speed > 0) {
                                carController.moveForward(speed)
                            } else if (speed < 0) {
                                carController.moveBackward(-speed)
                            } else {
                                carController.stop()
                            }
                        },
                        valueRange = -1f..1f,
                        modifier = Modifier
                            .height(10.dp)
                            .width(200.dp)
                            .padding(vertical = 16.dp)
                            .graphicsLayer(rotationZ = 270f)
                    )
                }
            }
        }

        // 设备列表对话框
        if (showDeviceList) {
            AlertDialog(
                onDismissRequest = { showDeviceList = false },
                title = { Text("选择设备") },
                text = {
                    // 设备列表
                    LazyColumn {
                        items(deviceList) { device ->
                            TextButton(
                                onClick = {
                                    onConnect(device)
                                    showDeviceList = false
                                },
                                modifier = Modifier.fillMaxWidth()
                            ) {
                                Column(
                                    modifier = Modifier.fillMaxWidth(),
                                    horizontalAlignment = Alignment.Start
                                ) {
                                    Text(device.device.name ?: "未知设备")
                                    Text(
                                        device.device.address,
                                        style = MaterialTheme.typography.bodySmall
                                    )
                                }
                            }
                        }
                    }
                },
                dismissButton = {
                    TextButton(onClick = { onStartScan() }) {
                        Text("扫描设备")
                    }
                },
                confirmButton = {
                    TextButton(onClick = { showDeviceList = false }) {
                        Text("取消")
                    }
                }
            )
        }
    }
}